function [o,p] = Aco()
        %set the pseudo random sequence, so the experiment is repeatable (and
        %all the students get the same results)
        s = RandStream('mt19937ar');
        RandStream.setDefaultStream(s);
        reset(s);
        
        %Init the structures
        [o,p] = Init();
        
        %Main loop
        start = p.tour        
        for tour = start : o.k
            p.tour = tour;
            
            currPaths = zeros( o.m, o.n );
            inPaths = zeros( o.m, o.n );
            currLens = zeros( o.m, 1 );
            
            %choose starting nodes          
            for ant = 1:o.m                                     
                currPaths( ant, 1 ) = floor( (o.n-1+1) * rand + 1 );
                inPaths( ant, currPaths( ant, 1 ) ) = 1;                
            end              
            
            %step-by-step go _simultaneously_ through graph
            for i = 2:o.n                            
                for ant = 1:o.m
                    
                    %fix current city
                    src = currPaths( ant, i-1 );
                    
                    %find next city
                    denum = 0;
                    for dst = 1 : o.n
                        if ( inPaths( ant, dst ) == 0 && dst ~= src ) 
                            denum = denum + ( p.pheromoneMatrix( src, dst )^o.alpha * (1/p.distMatrix( src, dst ) )^o.beta );                    
                        end                
                    end

                    roulette = rand;
                    left = 0;
                    right = 0;                    
                    for dst = 1 : o.n
                        if ( inPaths( ant, dst ) == 0 && dst ~= src ) 
                            left = right;
                            right = right + (p.pheromoneMatrix( src, dst )^o.alpha * (1/p.distMatrix( src, dst ) )^o.beta)/denum;
                            if (roulette > left && roulette <= right )                                
                                currPaths( ant, i ) = dst;
                                break
                            end
                        end                 
                    end 
                                                                            
                    %mark as used
                    dst = currPaths( ant, i ); 
                    inPaths( ant, dst ) = 1;
                                        
                    %update lenghts
                    currLens( ant, 1 ) = currLens( ant, 1 ) + p.distMatrix( src, dst );

                    
                    %local pheromone update
%--------------------------------------------------------------------------
                    %TODO. Rule One.
                    % Comment-out if necessery. Local pheromone update 
                    % on edge that just has been used.
                    p.pheromoneMatrix( src, dst ) = (1 - o.phi) * p.pheromoneMatrix( src, dst ) + o.phi * p.tauZero;
%--------------------------------------------------------------------------                    
                    
                    p.pheromoneMatrix( dst, src ) = p.pheromoneMatrix( src, dst );            
                    
                end
            end
            
            % add cost for closing the cycle
            for ant = 1:o.m
                currLens( ant, 1 ) = currLens( ant, 1 ) + p.distMatrix( currPaths( ant, o.n ), currPaths( ant, 1 ) );
            end
            
            % find best ant and remember it
            [bestLen, bestAnt] = min( currLens );                                    
            p.acoBestPaths( p.tour, :) = currPaths( bestAnt, : );   
            p.acoBestLens( p.tour ) = bestLen;
            
                       
            % global pheromone update                         
            
%--------------------------------------------------------------------------           
            %TODO.Rule Two.
            % Comment-out if necessery. Global pheromone update 
            % on every existing edge.
            p.pheromoneMatrix( :, : ) = (1 - o.rho) * p.pheromoneMatrix( :, : );
%--------------------------------------------------------------------------           

            for i = 1 : o.n
                if ( i == 1 )
                    src = currPaths( bestAnt, o.n );
                else
                    src = currPaths( bestAnt, i - 1 );
                end
                dst = currPaths( bestAnt, i );                                
%--------------------------------------------------------------------------           
                %TODO. Rule Three.
                % Comment-out if necessery. Global pheromone update
                % on edges belonging to the best path.
                p.pheromoneMatrix( src, dst ) = (1 - o.rho) * p.pheromoneMatrix( src, dst ) + o.rho / bestLen;
%--------------------------------------------------------------------------           

                p.pheromoneMatrix( dst, src ) = p.pheromoneMatrix( src, dst );
            end
            
            % show results inrementally 
            display(sprintf('%d %d', p.tour, p.acoBestLens(p.tour)));
        end 
        p.tour = p.tour+1;
end