function [ o, p ] = Init( )
    %init options
    o = initOptions();
    
    %read cities coordinates from file
    p.cities = GetCities( o.name );    
    
    %set o.n to denote how many cities are there
    o.n = length( p.cities );

%--------------------------------------------------------------------------    
    %TODO. Choose a good (means "typically used in TSP") value for number 
    %of ants:
    o.m = 1; %? i.e.: 1, 10, 100, o.tourMax, o.n, (o.n)^2, ...
%--------------------------------------------------------------------------    
              
    initProblem();
    

%HELP FUNCTIONS:
    
    function o = initOptions()                
        
        % instance file, let go in steps of mighty Ulysses...
        o.name   = 'ulysses22.tsp';

%--------------------------------------------------------------------------    
        %TODO. Play with parameters
        
        % o.alpha - how important is the pheromone trail
        o.alpha     = 1;
                
        % o.beta - how important is the distance between cities
        o.beta      = 2; 
                
        % o.phi - local pheromone update rate
        o.phi       = 0.1;
        
        % o.rho - global pheromone update rate
        o.rho       = 0.1;
        
        % o.k - number of tours (iterations of the ACO algorithm), that
        % is how many tours each ant is constructing
        o.k         = 20;
%--------------------------------------------------------------------------

    end %Init


    %Set the initialization values for the pheromone trail as sum of 
    %distances to closest city from each city 
    function len = getEstimatedBestLen()
        len = 0;
        for i = 1:o.n
            row = p.distMatrix(i, :);            
            len = len + min( row( row ~= 0 ) );                        
        end        
    end %getEstimatedBestLen


    %Inits arrays, etc.
    function initProblem()     
        %stores the best path found by the greedy algorithm
        p.greedyBestPath = zeros( o.n, 1 );        
        
        %stores the lenght of the best path found by the greedy algorithm
        p.greedyBestLen  = 0;                
        
        %stores best path for each iteration of Aco
        p.acoBestPaths   = zeros( o.k, o.n );
        
        %stores the lenght of best paths for each iteration of Aco
        p.acoBestLens    = zeros( o.k, 1 );                 
        
        %counter that indicates the current iteration of Aco
        p.tour           = 1;                               
        
        %matrix with distances between cities, computed before hand        
        p.distMatrix = zeros(o.n, o.n);
        for i = 1 : o.n
            for j = i : o.n
                p.distMatrix(i,j) = euclidanDist( p.cities(i,1), p.cities(i,2), p.cities(j,1), p.cities(j,2) );
                p.distMatrix(j,i) = p.distMatrix(i,j);
            end
        end                        
        
        %a very simple estimation of the best optimal solution
        p.estimatedBestLen = getEstimatedBestLen();        
        
        
        %initial pheromone level
        %For demonstration purposes (quicker convergance) we are 
        %putting less pheromone than ususally in Aco. 
        p.tauZero         = 1 / ( o.n * p.estimatedBestLen * 100 );
        
        %the matrix of pheromones between all cities
        p.pheromoneMatrix = ones( o.n, o.n ) .* (p.tauZero);                
                
        %different instances of TSP sometimes assume different way of 
        %treating the floating number, here we assume the right thing to do
        %is to round up all distances
        function dist = euclidanDist(x1,y1,x2,y2)
            dist = round( sqrt( (x1-x2)^2+(y1-y2)^2 ) );
            %dist = ceil( sqrt( (x1-x2)^2+(y1-y2)^2 ) );
            %dist = sqrt( (x1-x2)^2+(y1-y2)^2 );
        end
    end %initProblem

end %Init
